package de.unisaar.jfap.ex8;

import static org.assertj.core.api.Assertions.assertThat;

import de.unisaar.jfap.ex7.BinaryPredicate;
import de.unisaar.jfap.ex7.BinaryPredicate.EqualsPredicate;
import de.unisaar.jfap.ex7.DirectedGraph;
import de.unisaar.jfap.ex7.Edge;
import java.io.IOException;
import java.util.List;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class DijkstraShortestPathTest {

  DirectedGraph<Integer, String> graph;
  DijkstraShortestPath dsp;
  static BinaryPredicate<String> pred;

  @BeforeAll
  static void setup() {
    pred = new EqualsPredicate<>();
  }

  @BeforeEach
  void init() throws IOException {
    String graphString =
        String.format(
            "s --> w(2) z(4)%n"
                + "z --> w(5) y(9)%n"
                + "v --> w(1) s(6)%n"
                + "w --> x(6) q(3)%n"
                + "t --> u(1) v(9) s(8)%n"
                + "u --> t(5) v(7)%n"
                + "x --> z(8) u(7)%n"
                + "q --> x(2)%n"
                + "y --> x(5)%n"
                + "r --> s(1)");
    graph = DirectedGraph.readGraphWithWeights(graphString);
    dsp = new DijkstraShortestPath(graph);
  }

  @Test
  void testExistingShortestPath() {

    int source = graph.findVertex("s", pred);
    int target = graph.findVertex("u", pred);
    // path from 's' to 'u'
    List<Edge<Integer>> path = dsp.shortestPath(source, target);
    // print path
    System.out.format("%s", graph.getVertexProperty(0));
    for (Edge<Integer> edge : path) {
      System.out.format("--%s-->%s", edge.getInfo(), graph.getVertexProperty(edge.getEndVertex()));
    }
    System.out.println();
    // check path
    assertThat(path)
        .map(
            x ->
                graph.getVertexProperty(x.getStartVertex())
                    + graph.getVertexProperty(x.getEndVertex()))
        .containsExactly("sw", "wq", "qx", "xu");
  }

  @Test
  public void testStartEqualsEnd() {
    int source = graph.findVertex("s", pred);
    // from 's' to 's'
    List<Edge<Integer>> path = dsp.shortestPath(source, source);
    // check path
    assertThat(path.size()).isEqualTo(0);
  }

  @Test
  public void testNonExistingShortestPath() {
    int source = graph.findVertex("s", pred);
    int target = graph.findVertex("r", pred);
    // from 's' to 'r'
    List<Edge<Integer>> path = dsp.shortestPath(source, target);
    // check path
    assertThat(path).isNull();
  }
}
