package de.unisaar.jfap.ex8;

import static org.assertj.core.api.Assertions.assertThat;

import de.unisaar.jfap.ex7.DirectedGraph;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

class TarjanVisitorTest {

  DirectedGraph<String, String> graph;

  @BeforeEach
  void init() throws IOException {
    String graphString =
        String.format(
            "s --> z w%n"
                + "z --> y w%n"
                + "y --> x%n"
                + "x --> z%n"
                + "w --> q x%n"
                + "q --> x%n"
                + "t --> v u%n"
                + "u --> v t%n"
                + "v --> w s");
    graph = DirectedGraph.readGraph(graphString);
  }

  @Test
  void testTarjanVisitor() {
    // use DFS with Tarjan SCC visitor
    TarjanVisitor<String, String> visitor = new TarjanVisitor<>();
    graph.dfs(visitor);
    // print resulting SCCs
    int counter = 0;
    Set<Set<String>> labeledSccs = new HashSet<>();
    for (Set<Integer> comp : visitor.getSccs()) {
      System.out.format("Component %d: ", ++counter);
      Set<String> labeledComp = new HashSet<>();
      for (Integer v : comp) {
        String label = graph.getVertexProperty(v);
        System.out.format("%s ", label);
        labeledComp.add(label);
      }
      labeledSccs.add(labeledComp);
      System.out.println();
    }

    // check results
    assertThat(labeledSccs)
        .isEqualTo(
            Set.of(Set.of("s"), Set.of("w", "z", "y", "x", "q"), Set.of("v"), Set.of("t", "u")));
  }
}
